<?php

declare(strict_types = 1);

namespace Drupal\Tests\facets_form\Functional;

use Drupal\facets\Entity\Facet;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\facets\Functional\BlockTestTrait;
use Drupal\Tests\facets\Functional\ExampleContentTrait;
use Drupal\Tests\facets_form\Traits\FacetUrlTestTrait;

/**
 * Test the facets form.
 *
 * @group facets_form
 */
class IntegrationTest extends BrowserTestBase {

  use ExampleContentTrait;
  use BlockTestTrait;
  use FacetUrlTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'facets_form',
    'facets_search_api_dependency',
    'node',
    'search_api',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->setUpExampleStructure();
    $this->insertExampleContent();
    $this->assertSame(5, $this->indexItems('database_search_index'));
  }

  /**
   * Test the facets form.
   */
  public function testFacetsForm(): void {
    $assert = $this->assertSession();
    $page = $this->getSession()->getPage();
    $this->drupalLogin($this->drupalCreateUser(['administer blocks']));
    $this->createFacet('Emu', 'emu');
    $this->createFacet('Llama', 'llama');
    $facet = Facet::load('llama');
    $facet->setOnlyVisibleWhenFacetSourceIsVisible(FALSE);
    $facet->setWidget('facets_form_dropdown');
    $facet->save();
    // Place the Facets Form block for a view page source.
    $block = $this->drupalPlaceBlock('facets_form:search_api:views_page__search_api_test_view__page_1');
    $this->drupalGet('admin/structure/block/manage/' . $block->id());
    $config_form = $assert->elementExists('css', '.block-form');
    $assert->fieldValueEquals('Submit button', 'Search');
    $assert->fieldValueEquals('Reset button', 'Clear filters');
    $config_form->fillField('Submit button', 'Apply');
    $config_form->fillField('Reset button', 'Reset');
    $config_form->checkField('Llama');
    $config_form->pressButton('Save block');
    $this->drupalGet('search-api-test-fulltext');
    $assert->elementsCount('css', '.views-row', 5);
    $form = $assert->elementExists('css', 'form#facets-form');

    // The form contains only the widget from the module.
    $assert->elementExists('css', 'select#edit-llama--2', $form);
    $assert->elementNotExists('css', 'select#edit-emu--2', $form);
    $assert->elementNotExists('css', 'select#edit-alpaca--2', $form);

    // The form submits and filters the results.
    $page->selectFieldOption('llama[]', 'article');
    $assert->buttonExists('Apply', $form)->press();
    $this->assertCurrentUrl('search-api-test-fulltext', ['f' => ['llama:article']]);
    $assert->elementsCount('css', '.views-row', 2);

    $page->selectFieldOption('llama[]', 'item');
    $assert->buttonExists('Apply', $form)->press();
    $this->assertCurrentUrl('search-api-test-fulltext', ['f' => ['llama:item']]);
    $assert->elementsCount('css', '.views-row', 3);

    $page->clickLink('Reset');
    $this->assertCurrentUrl('search-api-test-fulltext');
    $assert->elementsCount('css', '.views-row', 5);

    // Test the CheckboxWidget widget.
    $facet->setWidget('facets_form_checkbox', ['indent_class' => 'super-indented']);
    $facet->save();
    // Pass an arbitrary query string in order to check its preservation.
    $this->drupalGet('search-api-test-fulltext', [
      'query' => [
        'foo' => 'bar',
        'baz' => ['qux', 'quux'],
      ],
    ]);
    // Check form submit without any filter.
    $form->pressButton('Apply');
    // Check query string preservation when submitting with no filter changes.
    $this->assertCurrentUrl('search-api-test-fulltext', [
      'foo' => 'bar',
      'baz' => ['qux', 'quux'],
    ]);
    $form->checkField('item');
    $form->pressButton('Apply');
    // Check query string preservation after submitting with filter changes.
    $this->assertCurrentUrl('search-api-test-fulltext', [
      'foo' => 'bar',
      'baz' => ['qux', 'quux'],
      'f' => ['llama:item'],
    ]);
    $assert->checkboxChecked('item', $form);
    $assert->elementsCount('css', '.views-row', 3);
    $form->checkField('article');
    $form->pressButton('Apply');
    // Check query string preservation after submitting with filter changes.
    $this->assertCurrentUrl('search-api-test-fulltext', [
      'foo' => 'bar',
      'baz' => ['qux', 'quux'],
      'f' => ['llama:article', 'llama:item'],
    ]);
    $assert->checkboxChecked('item', $form);
    $assert->checkboxChecked('article', $form);
    $assert->elementsCount('css', '.views-row', 5);
    // Check query string preservation after resetting the filters.
    $page->clickLink('Reset');
    $this->assertCurrentUrl('search-api-test-fulltext', [
      'foo' => 'bar',
      'baz' => ['qux', 'quux'],
    ]);

    // Change configured facets.
    $this->createFacet('Alpaca', 'alpaca');
    $facet = Facet::load('alpaca');
    $facet->setWidget('facets_form_dropdown');
    $facet->save();
    $this->drupalGet('admin/structure/block/manage/' . $block->id());
    $assert->fieldNotExists('Emu');
    $assert->fieldExists('Llama');
    $assert->fieldExists('Alpaca');
    $page->checkField('Alpaca');
    $page->uncheckField('Llama');
    $page->pressButton('Save block');
    $this->drupalGet('search-api-test-fulltext');
    $form = $assert->elementExists('css', 'form#facets-form');
    $assert->elementExists('css', 'select#edit-alpaca--2', $form);
    $assert->elementNotExists('css', 'select#edit-llama--2', $form);
    $assert->elementNotExists('css', 'select#edit-emu--2', $form);
  }

}
